#ifndef _DARWIN_ARM_KINEMATICS_H
#define _DARWIN_ARM_KINEMATICS_H

#include <kdl/chain.hpp>
#include <kdl/chainfksolverpos_recursive.hpp>
#include <kdl/chainiksolver.hpp>
#include <kdl/chainiksolverpos_nr_jl.hpp>
#include <kdl/chainiksolvervel_pinv.hpp>
#include <kdl/frames_io.hpp>

class CDarwinArmKinematics
{ 
  private:
    KDL::Chain *chain;
    KDL::JntArray q_max;
    KDL::JntArray q_min;
    KDL::ChainFkSolverPos_recursive *fksolver;
    KDL::ChainIkSolverVel_pinv *iksolver_vel;
    KDL::ChainIkSolverPos_NR_JL *iksolver_pos;
    unsigned int num_dof;
  protected:
    double angle_norm(double angle);
  public:
    CDarwinArmKinematics();
    void load_chain(std::string &filename);
    void get_forward_kinematics(std::vector<double> &angles,std::vector<double> &pos,std::vector<double> &rot);
    void get_inverse_kinematics(std::vector<double> &pos,std::vector<double> &rot,std::vector<double> &angles);
    ~CDarwinArmKinematics();
};

#endif
