#include "darwin_robot.h"
#include "darwin_robot_exceptions.h"

#include <iostream>

std::string robot_device="A603LOBS";
std::string config_file="../src/xml/darwin_action.xml";

int main(int argc, char *argv[])
{
  int i=0,j=0,num_v1_servos,num_v2_servos;
  std::vector<int> servos;
  std::vector<double> angles,speeds,accels;

  try{
    CDarwinRobot darwin("Darwin",robot_device,1000000,0xC0);
    darwin.mm_load_config(config_file);
    darwin.mm_get_num_servos(&num_v1_servos,&num_v2_servos);
    std::cout << "Found " << num_v1_servos << " v1 servos and " << num_v2_servos << " v2 servos" << std::endl;
    // execute the walk ready action
    std::cout << "Assign servos to the action module" << std::endl;
    darwin.mm_enable_power();
    for(i=1;i<=num_v1_servos;i++)
    {
      darwin.mm_enable_servo(i);
      darwin.mm_assign_module(i,DARWIN_MM_ACTION);
    }
    darwin.mm_start();
    // execute an action
    darwin.action_load_page(9);
    darwin.action_start();
    while(darwin.action_is_page_running())
    {
      std::cout << "Executing action ..." << std::endl;
      std::cout << "Period: " << darwin.mm_get_current_period() << " ms" << std::endl;
      usleep(100000);
    }
    // sit down
    darwin.action_load_page(15);
    darwin.action_start();
    while(darwin.action_is_page_running())
    {
      std::cout << "Executing action ..." << std::endl;
      std::cout << "Period: " << darwin.mm_get_current_period() << " ms" << std::endl;
      usleep(100000);
    }
    // sit up
    darwin.action_load_page(16);
    darwin.action_start();
    while(darwin.action_is_page_running())
    {
      std::cout << "Executing action ..." << std::endl;
      std::cout << "Period: " << darwin.mm_get_current_period() << " ms" << std::endl;
      usleep(100000);
    }
    /* walk forward for a while */
    darwin.mm_stop();
    for(i=1;i<=num_v1_servos;i++)
    {
      darwin.mm_enable_servo(i);
      darwin.mm_assign_module(i,DARWIN_MM_WALKING);
    }
    darwin.mm_start();
    darwin.walking_set_speeds(20,0,0);
    darwin.walking_start();
    sleep(2);
    std::cout << "Stop walking ..." << std::endl;
    darwin.walking_stop();
    while(darwin.is_walking())
    {
      std::cout << "Walking ..." << std::endl;
      std::cout << "Period: " << darwin.mm_get_current_period() << " ms" << std::endl;
      usleep(100000);
    }
    /* move joints */
    darwin.mm_stop();
    for(i=1;i<=num_v1_servos;i++)
    {
      darwin.mm_enable_servo(i);
      darwin.mm_assign_module(i,DARWIN_MM_JOINTS);
    }
    darwin.mm_start();
    servos.push_back(1);
    servos.push_back(3);
    servos.push_back(5);
    angles.push_back(45.0);
    angles.push_back(45.0);
    angles.push_back(45.0);
    speeds.push_back(10.0);
    speeds.push_back(10.0);
    speeds.push_back(10.0);
    accels.push_back(10.0);
    accels.push_back(10.0);
    accels.push_back(10.0);
    std::cout << "Start joint_motion ..." << std::endl;
    darwin.joints_load(servos,angles,speeds,accels);
    darwin.joints_start();
    while(darwin.are_joints_moving())
    {
      std::cout << "moving joints ..." << std::endl;
      std::cout << "Period: " << darwin.mm_get_current_period() << " ms" << std::endl;
      usleep(100000);
    }
    std::cout << "done" << std::endl;
    /* head tracking */
    darwin.mm_stop();
    for(i=19;i<=20;i++)
    {
      darwin.mm_enable_servo(i);
      darwin.mm_assign_module(i,DARWIN_MM_HEAD);
    }
    darwin.mm_start();
    darwin.head_tracking_set_pan_pid(0.04,0.0,0.0);
    darwin.head_tracking_set_tilt_pid(0.04,0.0,0.0);
    darwin.head_tracking_set_target(0.0,90.0);
    darwin.head_tracking_start();
    for(i=0;i<40;i++)
    {
      std::cout << "head tracking ..." << std::endl;
      std::cout << "Period: " << darwin.mm_get_current_period() << " ms" << std::endl;
      usleep(10000);
    }
    darwin.head_tracking_stop();
    darwin.mm_stop();
    for(i=1;i<=num_v1_servos;i++)
    {
      darwin.mm_enable_servo(i);
      darwin.mm_assign_module(i,DARWIN_MM_ACTION);
    }
    darwin.mm_start();
    darwin.action_load_page(15);
    darwin.action_start();
    while(darwin.action_is_page_running())
    {
      std::cout << "Executing action ..." << std::endl;
      std::cout << "Period: " << darwin.mm_get_current_period() << " ms" << std::endl;
      usleep(100000);
    }
    /* disable servos and read position */
    darwin.mm_stop();
    for(i=1;i<=num_v1_servos;i++)
    {
      darwin.mm_disable_servo(i);
      darwin.mm_assign_module(i,DARWIN_MM_NONE);
    }
    darwin.mm_start();
    for(j=0;j<1000;j++)
    {
      angles=darwin.mm_get_servo_angles();
      std::cout << "Period: " << darwin.mm_get_current_period() << " ms" << std::endl;
      for(i=0;i<angles.size();i++)
        std::cout << "Servo " << i << " angle: " << angles[i] << std::endl;
      usleep(100000);
    }
    // enable all servos and assign them to the action module
    darwin.mm_stop();
    darwin.mm_disable_power();
  }catch(CException &e){
    std::cout << e.what() << std::endl;
  }
}
